@@ -25,15 +25,23 @@
 
 #include <ccrtp/rtppkt.h>
 
+
 #define REPLAY_WINDOW_SIZE 64
 
 
-const int SrtpAuthenticationNull     = 0;
-const int SrtpAuthenticationSha1Hmac = 1;
+const int SrtpAuthenticationNull     =  0;
+const int SrtpAuthenticationSha1Hmac =  1;
+const int SrtpAuthenticationSkeinHmac = 2;
 
 const int SrtpEncryptionNull  = 0;
 const int SrtpEncryptionAESCM = 1;
 const int SrtpEncryptionAESF8 = 2;
+const int SrtpEncryptionTWOCM = 3;
+const int SrtpEncryptionTWOF8 = 4;
+
+#ifdef SRTP_SUPPORT
+#include <ccrtp/crypto/AesSrtp.h>
+#endif
 
 #ifdef CCXX_NAMESPACES
 namespace ost {
@@ -373,13 +381,24 @@ namespace ost {
 	    int32  n_s;
 	    uint8* k_s;
 
-	    uint8 ealg;
-	    uint8 aalg;
-	    uint8 ekeyl;
-	    uint8 akeyl;
-	    uint8 skeyl;
-	    uint8 tagLength;
+	    int32 ealg;
+	    int32 aalg;
+	    int32 ekeyl;
+	    int32 akeyl;
+	    int32 skeyl;
+	    int32 tagLength;
 	    bool  seqNumSet;
+
+        void*   macCtx;
+
+#ifdef SRTP_SUPPORT
+	    AesSrtp* cipher;
+	    AesSrtp* f8Cipher;
+#else
+	    void* cipher;
+	    void* f8Cipher;
+#endif
+
     };
 #ifdef  CCXX_NAMESPACES
 }
